<?php
require_once((defined("RHACO_DIR")?constant("RHACO_DIR"):"")."tag/data/SimpleTag.php");
require_once((defined("RHACO_DIR")?constant("RHACO_DIR"):"")."tag/TagParser.php");
/**
 * @author Kazutaka Tokushima
 * @license New BSD License
 * @copyright Copyright 2005- The Rhacophorus Project. All rights reserved.
 */
class TemplateParser extends TagParser{
	var $replaceList	= array();
	var $blockList	= array();

	function setReplace($arrayOrSource,$dest=""){
		if(!is_array($arrayOrSource)){
			$arrayOrSource = array($arrayOrSource=>$dest);
		}
		foreach($arrayOrSource as $key => $value){
			$this->replaceList[$key]	= $value;
		}
	}
	function setBlock(){
		foreach(func_get_args() as $templateFileName){
			if(is_array($templateFileName)){
				foreach($templateFileName as $value){
					$this->blockList[] = $value;
				}
			}else if(is_string($templateFileName)){
				$this->blockList[] = $templateFileName;
			}
		}
	}
	function _system1001_Extends($src){
		$tag			= new SimpleTag();
		$blockList	= array($this->fileName);
		$path		= Rhaco::templatepath($this->fileName);
		$bool		= false;

		while($tag->set($src,$this->_getTagName("extends"))){
			$path		= Url::parseAbsolute(dirname($path),$tag->getParameter("href"));
			$blockList[]	= $path;
			$src			= $this->_getTemplateSource($path);
			$bool		= true;
		}
		if($bool){
			$this->setFileName($path);
			array_pop($blockList);
			$this->blockList = array_reverse(Variable::toArray($blockList));
		}
		return $src;
	}
	function _system1002_Comment($src){
		$tag = new SimpleTag();

		while($tag->set($src,$this->_getTagName("comment"))){
			$src = str_replace($tag->getPlain(),"",$src);
		}
		return $src;
	}
	function _system1003_Template($src){
		$tag		= new SimpleTag();
		$value	= "";
		$bool	= false;

		while($tag->set($src,$this->_getTagName("template"))){
			$value	.= $tag->getRawValue();
			$bool	= true;
			$src		= str_replace($tag->getPlain(),"",$src);
		}
		if($bool){
			return $value;
		}
		return $src;
	}
	function _system1003_Block($src){
		$tag			= new SimpleTag();
		$blockList	= array();

		foreach(Variable::toArray($this->blockList) as $blockTemplateFile){
			$blockTemplateFile	= Rhaco::templatepath($blockTemplateFile);	
			$blockTemplateSrc	= $this->_getTemplateSource($blockTemplateFile);
			if(empty($blockTemplateSrc)){
				Logger::warning(Message::_("read bloc template file [{1}]",$blockTemplateFile));
			}else{
				$newtag = new SimpleTag("template",$blockTemplateSrc);

				foreach($newtag->getIn($this->_getTagName("block")) as $blocktag){
					$blockList[strtolower($blocktag->getParameter("name","name"))]	= $blocktag->getRawValue();
				}
				Logger::debug(Message::_("read bloc template file [{1}]",$blockTemplateFile));
			}
		}
		while($tag->set($src,$this->_getTagName("block"))){
			$id		= strtolower($tag->getParameter("name","name"));
			$value	= $tag->getRawValue();

			if(array_key_exists($id,$blockList)){
				$value = $blockList[$id];
			}
			$src = str_replace($tag->getPlain(),$value,$src);
		}
		if(sizeof($blockList) > 0){
			$src = $this->_call($src,"_toTag");
		}
		return $src;
	}
	function _system1004_Replace($src){
		$tag = new SimpleTag();

		while($tag->set($src,$this->_getTagName("replace"))){
			$src = str_replace($tag->getParameter("src"),$tag->getParameter("dest"),$src);			
			$src = str_replace($tag->getPlain(),"",$src);
		}
		krsort($this->replaceList);
		foreach($this->replaceList as $key => $source){
			$src = str_replace($key,$source,$src);
		}
		return $src;
	}
	function _system1005_ReplaceVariable($src){
		if(preg_match_all("/<([\w\d:_-]+)[\s\t][^>]*?contents=([\"\'])[^\\2]*?\\2[^>]*?>/i",$src,$tagList)){
			foreach($tagList[0] as $id => $plain){
				$tag	 = new SimpleTag();
				$tag	->set(substr($src,strpos($src,$plain)),$tagList[1][$id]);

				if($tag->isParameter("contents")){
					$plain	= $tag->getPlain();
					$var		= $tag->getParameter("contents");

					$tag->removeParameter("contents");
					$tag->setValue($var);
					$src = str_replace($plain,$tag->get(),$src);
				}
			}
		}
		return $src;
	}

	function _toTag1001_Loop($src){
		if(preg_match_all("/\{#loop\((.+)\)\}/i",$src,$matchList)){
			foreach($matchList[0] as $key => $match){
				$val			= "";
				$counter		= 0;

				foreach(split(",",$matchList[1][$key]) as $value){
					$param = "param";
					
					if($counter == 1){		$param = "var";		}
					else if($counter == 2){	$param = "limit";	}						
					else if($counter == 3){	$param = "offset";	}

					$val .= sprintf(" %s=\"%s\"",$param,$value);
					$counter++;
				}
				$src = str_replace($matchList[0][$key],"<".$this->_getTagName("loop").$val.">",$src);
			}
			$src = str_replace("{#endloop}","</".$this->_getTagName("loop").">",$src);
		}
		return $src;
	}
	function _toTag1002_Keyloop($src){
		if(preg_match_all("/\{#keyloop\((.+)\)\}/i",$src,$matchList)){
			foreach($matchList[0] as $key => $match){
				$val			= "";
				$counter		= 0;

				foreach(split(",",$matchList[1][$key]) as $value){
					$param = "param";
					
					if($counter == 1){		$param = "key";		}
					else if($counter == 2){	$param = "limit";	}						
					else if($counter == 3){	$param = "offset";	}

					$val .= sprintf(" %s=\"%s\"",$param,$value);
					$counter++;
				}
				$src = str_replace($matchList[0][$key],"<".$this->_getTagName("loop").$val.">",$src);
			}
			$src = str_replace("{#endkeyloop}","</".$this->_getTagName("loop").">",$src);
		}
		return $src;
	}
	function _toTag1003_For($src){
		if(preg_match_all("/\{#for\((.+)\)\}/i",$src,$matchList)){
			foreach($matchList[0] as $key => $match){
				$val			= "";
				$counter		= 0;

				foreach(split(",",$matchList[1][$key]) as $value){
					$param = "counter";
					
					if($counter == 1){		$param = "start";	}
					else if($counter == 2){	$param = "end";		}						
					else if($counter == 3){	$param = "step";		}

					$val .= sprintf(" %s=\"%s\"",$param,$value);
					$counter++;
				}
				$src = str_replace($matchList[0][$key],"<".$this->_getTagName("for").$val.">",$src);
			}
			$src = str_replace("{#endfor}","</".$this->_getTagName("for").">",$src);
		}
		return $src;
	}
	function _toTag1004_If($src){
		if(preg_match_all("/\{#if\((.+)\)\}/i",$src,$matchList)){
			foreach($matchList[0] as $key => $match){
				$val			= "";
				$counter		= 0;

				foreach(split(",",$matchList[1][$key]) as $value){
					if($counter == 0){		$param = "param";	}
					else if($counter == 1){	$param = "value";	}						

					$value = preg_replace("/^[\"]/","",$value);
					$value = preg_replace("/[\"]$/","",$value);
					$val .= sprintf(" %s=\"%s\"",$param,$value);
					$counter++;
				}
				$src = str_replace($matchList[0][$key],"<".$this->_getTagName("if").$val.">",$src);
			}
			$src = str_replace("{#else}","<".$this->_getTagName("else")." />",$src);
			$src = str_replace("{#endif}","</".$this->_getTagName("if").">",$src);
		}
		return $src;		
	}
	function _toTag1005_IfNot($src){
		if(preg_match_all("/\{#ifnot\((.+)\)\}/i",$src,$matchList)){
			foreach($matchList[0] as $key => $match){
				$val			= "";
				$counter		= 0;

				foreach(split(",",$matchList[1][$key]) as $value){
					if($counter == 0){		$param = "param";	}
					else if($counter == 1){	$param = "value";	}						
					$value = preg_replace("/^[\"]/","",$value);
					$value = preg_replace("/[\"]$/","",$value);					

					$val .= sprintf(" %s=\"%s\"",$param,$value);
					$counter++;
				}
				$src = str_replace($matchList[0][$key],"<".$this->_getTagName("ifnot").$val.">",$src);
			}
			$src = str_replace("{#else}","<".$this->_getTagName("else")." />",$src);
			$src = str_replace("{#endifnot}","</".$this->_getTagName("ifnot").">",$src);
		}
		return $src;
	}
	function _toTag1006_Block($src){
		if(preg_match_all("/\{#block\((.+)\)\}/i",$src,$matchList)){
			foreach($matchList[0] as $key => $match){
				$value	= $matchList[1][$key];
				$value	= preg_replace("/^[\"]/","",$value);
				$value	= preg_replace("/[\"]$/","",$value);
				$val		= sprintf(" %s=\"%s\"","name",$value);
				$src		= str_replace($matchList[0][$key],"<".$this->_getTagName("block").$val.">",$src);
			}
			$src = str_replace("{#endblock}","</".$this->_getTagName("block").">",$src);
		}
		return $src;		
	}	
		
	function _exec1001_Loop($src){
		$tag = new SimpleTag();

		while($tag->set($src,$this->_getTagName("loop"))){
			$class	= $this->_parsePlainVariable($tag->getParameter("param","param"));
			$id		= $tag->getParameter("var","var");
			$classnm	= $this->_variableQuote($class);
			$counter	= $this->_getVariableString($tag->getParameter("counter","counter"));
			$key		= $tag->getParameter("key","key");

			$count	= $this->_getVariableString("counter_".$classnm);
			$offset	= $tag->getParameter("offset","1");
			$limit	= $tag->getParameter("limit","0");

			$offsetName	= sprintf("\$_OFFSET_%s",$classnm);
			$limitName	= sprintf("\$_LIMIT_%s",$classnm);

			$function	= sprintf("%s%s = 1;%s",$this->_getPhpTagStart(),$count,$this->_getPhpTagEnd());
			$function	.= sprintf("%s %s = %d;%s",$this->_getPhpTagStart(),$offsetName,$offset,$this->_getPhpTagEnd());
			$function	.= sprintf("%s %s = %d;%s",$this->_getPhpTagStart(),$limitName,(($limit>0)?($offset + $limit):0),$this->_getPhpTagEnd());
			$function	.= sprintf("%sif(is_array(%s)):%s",$this->_getPhpTagStart(),$this->_getVariableString($class),$this->_getPhpTagEnd());
			$function	.= sprintf("%sforeach(%s as %s => %s):%s",$this->_getPhpTagStart(),$this->_getVariableString($class),$this->_getVariableString($key),$this->_getVariableString($id),$this->_getPhpTagEnd());
			$function	.= sprintf("%sif(%s <= %s):%s",$this->_getPhpTagStart(),$offsetName,$count,$this->_getPhpTagEnd());
			$function	.= sprintf("%s%s = %s;%s",$this->_getPhpTagStart(),$counter,$count,$this->_getPhpTagEnd());			
			$function	.= sprintf("%s",$tag->getRawValue());
			$function	.= sprintf("%sendif;%s",$this->_getPhpTagStart(),$this->_getPhpTagEnd());
			$function	.= sprintf("%s%s++;%s",$this->_getPhpTagStart(),$count,$this->_getPhpTagEnd());
			$function	.= sprintf("%sif(%s > 0 && %s >= %s){break;}%s",$this->_getPhpTagStart(),$limitName,$count,$limitName,$this->_getPhpTagEnd());
			$function	.= sprintf("%sendforeach;%s",$this->_getPhpTagStart(),$this->_getPhpTagEnd());
			$function	.= sprintf("%sendif;%s",$this->_getPhpTagStart(),$this->_getPhpTagEnd());

			$src = str_replace($tag->getPlain(),$function,$src);
		}
		return $src;
	}
	function _exec1002_For($src){
		$tag = new SimpleTag();

		while($tag->set($src,$this->_getTagName("for"))){
			$counterName	= $this->_getVariableString($tag->getParameter("counter","counter"));
			$start		= $this->_parsePlainVariable($tag->getParameter("start",0));
			$end			= $this->_parsePlainVariable($tag->getParameter("end",10));
			$step		= $this->_parsePlainVariable($tag->getParameter("step",1));
			
			$function = sprintf("%s for(%s=%s;%s<=%s;%s+=%s): %s",
								$this->_getPhpTagStart(),								
								$counterName,
								$start,
								$counterName,
								$end,
								$counterName,
								$step,
								$this->_getPhpTagEnd()
							);
			$function .= $tag->getRawValue();
			$function .= sprintf("%s endfor; %s",$this->_getPhpTagStart(),$this->_getPhpTagEnd());
			
			$src = str_replace($tag->getPlain(),$function,$src);
		}		
		return $src;
	}
	function _exec1003_If($src){
		$tag = new SimpleTag();
		
		while($tag->set($src,$this->_getTagName("if"))){
			$arg1		= $this->_parsePlainVariable($tag->getParameter("param"));
			$value		= $tag->getRawValue();
			$pattern		= "";
			$parse		= array();
			
			if(!is_bool($arg1) && !preg_match("/^[\$\'\"]/",$arg1)){
				$arg1 = sprintf("'%s'",$arg1);
			}
			if($tag->isParameter("value")){
				$arg2 = $this->_parsePlainVariable($tag->getParameter("value"));							
				if(!preg_match("/^([\$\'\"])/",$arg2)){
					$arg2 = sprintf("'%s'",$arg2);
				}
				$pattern = sprintf("%s == %s",$arg1,$arg2);
			}else{
				$pattern = sprintf("%s",$arg1);
			}
			$function	= sprintf("%s if(%s): %s",$this->_getPhpTagStart(),$pattern,$this->_getPhpTagEnd());
			
			if(preg_match("/(<[\s\t]*".$this->_getTagName("else")."[^>]*[\s\t]*\/>)/i",$tag->getRawValue(),$parse)){
				$value	= str_replace($parse[1],
								sprintf("%s else: %s",$this->_getPhpTagStart(),$this->_getPhpTagEnd()),
								$value
							);
			}
			$function	.= sprintf("%s",$value);
			$function	.= sprintf("%s endif; %s",$this->_getPhpTagStart(),$this->_getPhpTagEnd());
			
			$src = str_replace($tag->getPlain(),$function,$src);			
		}
		return $src;
	}
	function _exec1004_IfNot($src){
		$tag = new SimpleTag();

		while($tag->set($src,$this->_getTagName("ifnot"))){
			$arg1		= $this->_parsePlainVariable($tag->getParameter("param"));
			$value		= $tag->getRawValue();
			$pattern		= "";
			$parse		= array();

			if(!is_bool($arg1) && !preg_match("/^([\$\'\"])/",$arg1)){
				$arg1 = sprintf("\"%s\"",$arg1);
			}
			if($tag->isParameter("value")){
				$arg2 = $this->_parsePlainVariable($tag->getParameter("value"));							
				if(!preg_match("/^([\$\'\"])/",$arg2)){
					$arg2 = sprintf("\"%s\"",$arg2);
				}
				$pattern = sprintf("%s != %s",$arg1,$arg2);
			}else{
				$pattern = sprintf("!%s",$arg1);
			}
			$function	= sprintf("%s if(%s): %s",$this->_getPhpTagStart(),$pattern,$this->_getPhpTagEnd());
			
			if(preg_match("/(<[\s\t]*".$this->_getTagName("else")."[^>]*[\s\t]*\/>)/i",$tag->getRawValue(),$parse)){
				$value	= str_replace($parse[1],
								sprintf("%s else: %s",$this->_getPhpTagStart(),$this->_getPhpTagEnd()),
								$value
							);
			}
			$function	.= sprintf("%s",$value);
			$function	.= sprintf("%s endif; %s",$this->_getPhpTagStart(),$this->_getPhpTagEnd());

			$src = str_replace($tag->getPlain(),$function,$src);
		}
		return $src;
	}
	function _exec1005_invalid($src){
		$tag = new SimpleTag();

		while($tag->set($src,$this->_getTagName("invalid"))){
			$name		= $this->_parsePlainVariable($tag->getParameter("name","exceptions"));
			$errors		= $this->_parsePlainVariable($tag->getParameter("errors","errors"));
			$join		= $this->_parsePlainVariable($tag->getParameter("join","\n"));
			$var		= $this->_parsePlainVariable($tag->getParameter("var","var"));			
			$function	= $tag->getPlain();

			if(!empty($name)){
				$function	= sprintf("%sif(ExceptionTrigger::invalid(\"%s\")):%s",$this->_getPhpTagStart(),$name,$this->_getPhpTagEnd());
				$function	.= sprintf("%s%s = \"\";%s",$this->_getPhpTagStart(),$this->_getVariableString($errors),$this->_getPhpTagEnd());
				$function	.= sprintf("%s\$_INVALID_ = array();%s",$this->_getPhpTagStart(),$this->_getPhpTagEnd());
				$function	.= sprintf("%sforeach(ExceptionTrigger::get(\"%s\") as \$_INVALID_EXCEPTION):%s",$this->_getPhpTagStart(),$name,$this->_getPhpTagEnd());
				$function	.= sprintf("%s\$_INVALID_[] = \$_INVALID_EXCEPTION->getMessage();%s",$this->_getPhpTagStart(),$this->_getPhpTagEnd());
				$function	.= sprintf("%sendforeach;%s",$this->_getPhpTagStart(),$this->_getPhpTagEnd());
				$function	.= sprintf("%s%s = \$_INVALID_; %s",$this->_getPhpTagStart(),$this->_getVariableString($var),$this->_getPhpTagEnd());				
				$function	.= sprintf("%s%s = implode(html_entity_decode(\"%s\"),\$_INVALID_); %s",$this->_getPhpTagStart(),$this->_getVariableString($errors),trim(addslashes($join)),$this->_getPhpTagEnd());	
				$function	.= sprintf("%s",$tag->getRawValue());
				$function	.= sprintf("%sendif;%s",$this->_getPhpTagStart(),$this->_getPhpTagEnd());
			}
			$src = str_replace($tag->getPlain(),$function,$src);
		}
		return $src;
	}
	function _getTagName($value){
		return sprintf("RT:%s",$value);
	}
	function _getCacheUrl($templateFileName){
		return parent::_getCacheUrl(array_merge(Variable::toArray($templateFileName),Variable::toArray($this->blockList)));
	}
}
?>